#include "counter.h"

void init_counter(counter_t *c, int threshold, int num_cpu) {
  c->num_cpu = num_cpu;
  c->threshold = threshold;
  c->global = 0;
  c->local = malloc(num_cpu * sizeof(long));
  c->llock = malloc(num_cpu * sizeof(pthread_mutex_t));
  pthread_mutex_init(&c->glock, NULL);
  int i;
  for (i = 0; i < num_cpu; i++) {
    c->local[i] = 0;
    pthread_mutex_init(&c->llock[i], NULL);
  }
}

void update_counter(counter_t * c, int t_id, int amount) {
int cpu = t_id % c->num_cpu;
    pthread_mutex_lock(&c->llock[cpu]);
    c->local[cpu] += amount;
    if (c->local[cpu] >= c->threshold) {
        pthread_mutex_lock(&c->glock);
        c->global += c->local[cpu];
        pthread_mutex_unlock(&c->glock);
        c->local[cpu] = 0;
    }
    pthread_mutex_unlock(&c->llock[cpu]);
}

int get_counter_val(counter_t *c) {
    pthread_mutex_lock(&c->glock);
    int val = c->global;
    pthread_mutex_unlock(&c->glock);
    return val;
}

void sync_counter(counter_t *c) {
    int i;
    for (i = 0; i < c->num_cpu; i++) {
        pthread_mutex_lock(&c->llock[i]);
        pthread_mutex_lock(&c->glock);
        c->global += c->local[i];
        pthread_mutex_unlock(&c->glock);
        c->local[i] = 0;
        pthread_mutex_unlock(&c->llock[i]);
    }

}

void destroy_counter(counter_t *c) {
    pthread_mutex_destroy(&c->glock);
    int i;
    for (i = 0; i < c->num_cpu; i++) {
        pthread_mutex_destroy(&c->llock[i]);
    }
    free(c->local);
    free(c->llock);
}
