---
title: Recap I
---

# Recap & Discussion

### Concurrency

- What is concurrency?
- What are the essentials?
- What are the models for concurrency programming?

<v-click>

<br />

### Concurrency with Threads 

</v-click>

<v-clicks>

- What are threads?
- What information does a thread hold? What are shared among threads, what not?
- What kind of issues will we face when we use threads? What is the root of these issues?
- How to design your program using threads?

</v-clicks>

<br />

<v-clicks>

### Reading

See: [MIT 6.005 - Concurrency](http://web.mit.edu/6.005/www/fa16/classes/19-concurrency/)

</v-clicks>

---
title: Recap II
layout: two-cols
---

#### Memory Model


<div class="container flex justify-center mt-10">
    <img src="/images/thread-memory-model.png" class="block w-sm"/>
</div>

::right::

All of these threads are independently executing the same program, and they all share the same global memory, including the initialized data, uninitialized data, and heap segments.

A traditional UNIX process is simply a special case of a multithreaded processes; it is a process that contains just one thread.

#### Advantages over Processes

<v-clicks>

- Sharing information between threads is easy and fast.
- Thread creation is faster than process creation—typically, ten times faster or better. (On Linux, threads are implemented using the `clone()` system call)

</v-clicks>

---
title: Recap III
---


#### Pthreads

To compile them, you must include the header `pthread.h` in your code. On the link line, you must also explicitly link with the `pthreads` library, by adding the `-pthread` flag.

```sh
gcc -o main main.c -Wall -pthread
```

#### Thread Creation

```c
// Create a new thread, threads are peers and non-hierarchical
// Which means that you can create threads within threads
int pthread_create(pthread_t* thread, const pthread_attr_t *attr,
                   void* (*start_routine)(void*), void *arg);
```

#### Thread Completion

In case you want to wait for your threads to complete. _Why do we have a pointer to a pointer here?_

```c
// Waits for the thread identified by thread to terminate.
// You can even wait for a thread inside a thread's start_routine
// If a thread exits but is not joined, it is a "zombie thread".
int pthread_join(pthread_t thread, void ** value_ptr);
```

---
title: Recap IV
---

#### An Example

```c
typedef struct { int a; int b; } arg_t;
typedef struct { int x; int y; } ret_t;

void * mythread(void * arg) {
    ret_t * rvals = malloc(sizeof(ret_t));
    rvals->x = 1;
    rvals->y = 2;
    return (void * ) rvals; 
}

int main(int argc, char * argv[]) {
    pthread_t p;
    ret_t * rvals;
    arg_t args = { 10, 20 };
    if (pthread_create(&p, NULL, mythread, &args)) { return -1; }
    if (pthread_join(p, (void **) &rvals)) { return -1; }
    printf("returned %d %d\n", rvals->x, rvals->y);
    free(rvals);
    return 0;
}
```

Why are we using `malloc` and `free` here?

---
title: Recap V
---

### Other Useful Pthreads APIs

#### Thread IDs

```c
// Returns the thread ID of the calling thread
// Thread IDs are used by other Pthreads functions
pthread_t pthread_self(void); 

// Check whether two thread IDs are the same.
int pthread_equal(pthread_t t1, pthread_t t2);
```

#### Thread Termination

```c
// equivalent to return
// except it can also be called by functions that are called by the start_routine
void pthread_exit(void *retval); 
```

#### Detaching Thread

By default, a thread is **joinable**, meaning that when it terminates, another thread can obtain its return status using `pthread_join()`. Sometimes, we don’t care about the thread’s return status; we simply want the system to automatically clean up and remove the thread when it terminates.

```c
int pthread_detach(pthread_t thread);
// A thread can detach itself in the start_routine
pthread_detach(pthread_self());
```

---
title: Recap VI
layout: two-cols
---

### Thread Synchronization

To avoid race conditions, we must use a _mutex_(short for mutual exclusion) to ensure that only one thread a time can access the shared data.

A mutex has two states: _locked_ and _unlocked_.

When a thread locks a mutex, it becomes the owner of that mutex. Only the mutex owner can unlock the mutex.

**In general, we employ a different mutex for each shared resource.**

::right::

<div class="container flex justify-center mt-10">
    <img src="/images/mutex.png" class="block w-sm"/>
</div>


---
title: Recap VII
---

#### Statically Allocated Mutexes

A mutex can either be allocated as a static variable or be created dynamically at run time (for example using `malloc()`).

A mutex is a variable of the type `pthread_mutex_t`. Before it can be used, a mutex must always be initialized.

```c
pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;
```

<br/>

#### Dynamically Initializing a Mutex

The static initializer value `PTHREAD_MUTEX_INITIALIZER` can be used only for initializing a statically allocated mutex with default attributes.
 
```c
int pthread_mutex_init(pthread_mutex_t *mutex, const pthread_mutexattr_t *attr);
```

---
title: Recap VIII
---

#### Locking and Unlocking a Mutex

After initialization, a mutex is unlocked. You can use the following functions to lock and unlock the mutex:

```c
int pthread_mutex_lock(pthread_mutex_t *mutex);
int pthread_mutex_unlock(pthread_mutex_t *mutex);
```

**They always appear in pairs.**

#### Destroying a Mutex

When an automatically or dynamically allocated mutex is no longer required, it should be destroyed using `pthread_mutex_destroy()`.

```c
int pthread_mutex_destroy(pthread_mutex_t *mutex);
```

**Note:** It is not necessary to call `pthread_mutex_destroy()` on a mutex that was statically initialized using `PTHREAD_MUTEX_INITIALIZER`.


